''
'' cog storage command handler (no counters)
''
''        Author: Marko Lukat
'' Last modified: 2012/07/06
''       Version: 0.1
''
'' The command handler stays in the user area and uses setup..outb for storage.
'' Each transfer will move the entire block into or out of cog RAM. The transfer
'' runs at max hub access speed.
''
'' long[par][0]: cog startup:        !Z =    32 -> zero (ready)
''               transaction: insn:addr = 16:16 -> zero (transaction done)
'' long[par][1]: unused
'' long[par][2]: unused
'' long[par][3]: unused
''
'' acknowledgements
'' - code based on work done by Phil Pilgrim (PhiPi)
''
OBJ
  system: "core.con.system"
  
PUB null
'' This is not a top level object.

PUB init(ID, storage_setup)

  return system.launch(ID, @storage, storage_setup)

DAT             org     0                       ' cog binary header

header_2048     long    system#ID_2             ' magic number for a cog binary
                word    header_size             ' header size
                word    %00000000_00000000      ' flags
                word    0, 0                    ' start register, register count

header_size     fit     16
                
CON
  CMD_BGET = %000010_000 << 23                  ' if_never wrlong 0, 0
  CMD_BPUT = %000010_001 << 23                  ' if_never rdlong 0, 0

  MAX_SIZE = $1E6                               ' available storage (setup..outb), 2n

DAT             org     0

storage         jmpret  $, #setup               ' once

                wrlong  zero, par               '  +0 = setup/transaction done
                rdlong  addr, par wz            '  +0 = get command (insn:zero:addr = 9:7:16)
        if_z    jmp     #$-1                    '  +8

                ror     addr, #23{align}        '  -4   prepare for movi
                movi    sub7, addr              '  +0 = move opcode into insn
                movi    sub1, addr              '  +4   move opcode into insn

                shr     addr, #9                '  +8   extract hub location
                add     addr, mask              '  -4   add magic marker

sub7            hubop   $1F5, addr              ' |
                sub     sub7, dst2              ' |
                sub     addr, i2s7 wc           ' |
sub1            hubop   $1F4, addr              ' |
                sub     sub1, dst2              ' |
        if_nc   djnz    addr, #sub7             ' sub #7/djnz (Thanks Phil!)

                jmp     #$1FC                   ' restore initial settings

setup           mov     mask, move+0            ' |
                mov     phsa, move+1            ' |
                mov     phsb, move+2            ' |
                mov     dst2, move+3            ' |
                mov     i2s7, move+4            ' move overhead out of harms way

                jmp     %%0                     ' ret

' initialised data and/or presets

move            long    (MAX_SIZE - 2) << 23 | MAX_SIZE << 2 -1
                movd    sub7, #$1F5
                movd    sub1, #$1F4
                long    2 << 9                  ' dst +/-= 2
                long    2 << 23 | 7
                
' uninitialised data and/or temporaries

tail            fit
                
CON
  addr = $1F7   ' dirb
  dst2 = $1FA   ' frqa
  i2s7 = $1FB   ' frqb
  zero = $1FE   ' vcfg
  mask = $1FF   ' vscl
  
DAT